package com.eksad.masterdata.controller.apidgi.invoicenjbnsc;

import com.eksad.masterdata.service.invoicenjbnsc.DGIInvoiceNjbNscService;
import com.eksad.ddms.common.h2.dto.apigateway.request.PullNotaJBSCRequestDTO;
import com.eksad.ddms.common.h2.dto.ui.ResponseErrorDTO;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import lombok.extern.slf4j.Slf4j;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.Locale;
import java.util.logging.Level;
import java.util.logging.Logger;

@RestController
@Slf4j
public class DGIInvoiceNjbNscRESTController {

    @Autowired
    DGIInvoiceNjbNscService pullService;

    @Value("${dam.token.key.path}")
    private String TOKEN;

    @Value("${dam.masterdata.h1}")
    private String md_h1_url;

    @Value("${dam.portal.daya.path}")
    private String PORTAL_DAM_URL;

    @Value("${dam.portal.daya.inv}")
    private String PORTAL_DAM_INV_URL;

    @Value("${daw.portal.daya.inv}")
    private String PORTAL_DAW_INV_URL;

    @Value("${md.portal.profile}")
    public String MD_PROFILE;

    @RequestMapping(value = "/dgi-api/v1/inv2/read",
            method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Object> pullNota(@RequestHeader("GvtDealerId") String dealerid, @RequestBody PullNotaJBSCRequestDTO dto) throws ParseException {
        try {
            System.out.println(new ObjectMapper().writeValueAsString(dto));
            pullService.exceptionHelper(dto.getFromTime(), "yyyy-MM-dd HH:mm:ss", "fromtime");
            pullService.exceptionHelperNotMandatory(dto.getToTime(), "yyyy-MM-dd HH:mm:ss", "totime");

            if (pullService.isNotValidDate(dto.getFromTime())) {
                throw new Exception("format toTime tidak sesuai, seharusnya YYYY-MM-DD hh:mm:ss");
            } else if (pullService.isNotValidDate(dto.getToTime())) {
                throw new Exception("format toTime tidak sesuai, seharusnya YYYY-MM-DD hh:mm:ss");
            }

            pullService.checkToTimeIs7Days(dto.getFromTime(), dto.getToTime());

            pullService.dealerValidation(dealerid, dto.getDealerId());
        } catch (Exception ex) {
            Logger.getLogger(DGIInvoiceNjbNscRESTController.class.getName()).log(Level.SEVERE, null, ex);
            ResponseErrorDTO res = new ResponseErrorDTO();
            res.setMessage(ex.getMessage());
            res.setStatus(HttpStatus.BAD_REQUEST.value());
            res.setError(HttpStatus.BAD_REQUEST.getReasonPhrase());
            return ResponseEntity.badRequest().body(res);
        }
        return pullService.postRequestPullNJBNSC(dto);
    }



}
