package com.eksad.masterdata.service.invoicenjbnsc;

import com.eksad.ddms.common.h2.dto.apigateway.request.PullNotaJBSCRequestDTO;
import com.eksad.ddms.common.uri.apigateway.APIGatewayURI;
import com.eksad.masterdata.common.RequestUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.Locale;

@Service
public class DGIInvoiceNjbNscService {
    DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss", Locale.getDefault());

    private RestTemplate restTemplate = new RestTemplate();

    @Value("${dam.masterdata.h2}")
    private String md_h2_url;

    @Value("${dam.gigr.h3}")
    private String gigr_h3_url;

    @Value("${dam.custpurch.h3}")
    private String custpurch_h3_url;

    @Autowired
    private RequestUtil requestUtil;

    public ResponseEntity<Object> postRequestPullNJBNSC(PullNotaJBSCRequestDTO dto) throws ParseException {
        Object data = pullNJBNSC(dto);

        if(data != null){
            return ResponseEntity.status(HttpStatus.OK).body(data);
        }
        return null;
    }

    private Object pullNJBNSC(PullNotaJBSCRequestDTO dto) {
        ResponseEntity<Object> response = restTemplate.exchange(
                md_h2_url
                        + APIGatewayURI.PULL_NSC_NJB.getUri(),
                HttpMethod.POST,
                new HttpEntity<>(dto, requestUtil.getPreFormattedRequestWithToken().getHeaders()),
                Object.class);
        return response.getBody();
    }

    public void checkToTimeIs7Days(String fromTime, String toTime) throws Exception {
        DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date from = dateFormat.parse(fromTime);
        Date to = dateFormat.parse(toTime);
        LocalDateTime fromLocal = from.toInstant().atZone(ZoneId.of("Asia/Jakarta")).toLocalDateTime();
        LocalDateTime toLocal = to.toInstant().atZone(ZoneId.of("Asia/Jakarta")).toLocalDateTime();
        Duration durasi = Duration.between(fromLocal, toLocal);
        long dayDifference = Math.abs(durasi.toHours());

        if (dayDifference > (7 * 24) + 1) {  // if duration greater than 24*7 + 1 hours
            throw new Exception("tenggat waktu hanya diperbolehkan 7 hari");
        }
    }

    public void exceptionHelper(String time, String format, String message) throws Exception {
        if (time == null || time.trim().isEmpty()) {
            throw new Exception(message + " tidak boleh kosong");
        }

        try {
            SimpleDateFormat dateFormat = new SimpleDateFormat(format);
            dateFormat.parse(time);
        } catch (ParseException e) {
            throw new Exception("format " + message.concat(" tidak sesuai, seharusnya ").concat(format));
        }

    }

    public void exceptionHelperNotMandatory(String time, String format, String message) throws Exception {
        if (time != null && !time.trim().isEmpty()) {
            try {
                SimpleDateFormat dateFormat = new SimpleDateFormat(format);
                dateFormat.parse(time);
            } catch (ParseException e) {
                throw new Exception("format " + message.concat(" tidak sesuai, seharusnya ").concat(format));
            }
        }
    }

    public boolean isNotValidDate(String input) {
        try {
            dateTimeFormatter.parse(input);
            return false;
        } catch (Exception e) {
            return true;
        }
    }

    public void dealerValidation(String dealerHeader, String dealerRequest) throws Exception {
        if (dealerHeader != null && dealerHeader != dealerRequest) {
            if (!dealerHeader.equalsIgnoreCase(dealerRequest)) {
                throw new Exception("gagal dealerId " + dealerRequest + " tidak sesuai");
            }
        }
    }
}
